// stores/user.ts - 管理用户信息、登录状态、角色权限等
import { defineStore } from 'pinia'
import Taro from '@tarojs/taro'
import { userAPI, type UserInfo} from '@/services/api/user'
import { ref, computed } from 'vue'

export type UserRole = 'player' | 'hoster'

export const useUserStore = defineStore('user', () => {
  // 基本状态
  const openid = ref('')
  const nickname = ref('')
  const avatar = ref('')
  const role = ref<UserRole>('player')
  const isRegistered = ref(false)
  const currentRoom = ref<string | null>(null)
  const currentGameId = ref<string | null>(null)
  const loading = ref(false)
  const error = ref<string | null>(null)
  
  // 计算属性
  const userInfo = computed<UserInfo>(() => ({
    openid: openid.value,
    nickname: nickname.value,
    avatar: avatar.value,
    role: role.value
  }))
  
  // 保存用户状态到本地存储
  function saveUserToStorage() {
    const userData = {
      openid: openid.value,
      nickname: nickname.value,
      avatar: avatar.value,
      role: role.value,
      currentRoom: currentRoom.value,
      isRegistered: isRegistered.value,
    }
    
    Taro.setStorageSync('userInfo', JSON.stringify(userData))
    return userData
  }
  
  // 从本地存储加载用户状态
  function loadUserFromStorage(): boolean {
    try {
      const userInfo = Taro.getStorageSync('userInfo')
      if (!userInfo) return false
      
      const data = JSON.parse(userInfo)
      openid.value = data.openid || ''
      nickname.value = data.nickname || ''
      avatar.value = data.avatar || ''
      role.value = data.role || 'player'
      isRegistered.value = !!data.isRegistered
      currentRoom.value = data.currentRoom || null
      
      return true
    } catch (error) {
      console.error('加载用户信息失败:', error)
      return false
    }
  }
  
  // 初始化用户信息
  async function initUser() {
    loading.value = true
    error.value = null
    
    try {
      // 1. 尝试从本地存储加载
      const localDataLoaded = loadUserFromStorage()
      
      // 2. 如果本地有数据且已注册，直接返回成功
      if (localDataLoaded && isRegistered.value) {
        return true
      }
      
      // 3. 如果本地有openid但未注册，保持现状
      if (localDataLoaded && openid.value && !isRegistered.value) {
        return true
      }
      
      // 4. 否则，进行静默登录获取openid
      return await silentLogin()
    } catch (e) {
      console.error('初始化用户失败:', e)
      error.value = '初始化用户失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 静默登录 - 仅获取openid
  async function silentLogin() {
    loading.value = true
    error.value = null
    
    try {
      const loginRes = await Taro.login()
      if (!loginRes.code) {
        error.value = '登录失败'
        return false
      }
      
      const result = await userAPI.getOpenId(loginRes.code)
      if (!result.success || !result.data) {
        error.value = result.message || '获取OpenID失败'
        return false
      }
      
      // 设置openid
      openid.value = result.data.openid
      saveUserToStorage()
      return true
    } catch (e) {
      console.error('静默登录失败:', e)
      error.value = '静默登录失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 注册用户 - 获取用户信息
  async function registerUser() {
    loading.value = true
    error.value = null
    
    try {
      // 确保有openid
      if (!openid.value) {
        const loginSuccess = await silentLogin()
        if (!loginSuccess) return false
      }
      
      // 微信环境使用getUserProfile
      if (process.env.TARO_ENV === 'weapp') {
        const profileRes = await Taro.getUserProfile({
          desc: '用于完善用户资料'
        })
        
        if (!profileRes.userInfo) {
          error.value = '获取用户信息失败'
          return false
        }
        
        // 更新本地状态
        nickname.value = profileRes.userInfo.nickName
        avatar.value = profileRes.userInfo.avatarUrl
        isRegistered.value = true
        
        // 保存到本地存储
        saveUserToStorage()
        
        // 同步到云端 - 使用Service层处理
        await userAPI.saveUserToCloud({
          openid: openid.value,
          nickname: nickname.value,
          avatar: avatar.value,
          role: role.value
        })
        
        return true
      } else {
        // 非微信环境模拟注册
        nickname.value = '模拟用户_' + Date.now().toString().slice(-4)
        avatar.value = 'https://example.com/avatar.jpg'
        isRegistered.value = true
        
        // 保存到本地
        saveUserToStorage()
        return true
      }
    } catch (e) {
      console.error('用户注册失败:', e)
      error.value = '用户注册失败'
      return false
    } finally {
      loading.value = false
    }
  }
  
  // 设置用户角色
  function setRole(newRole: UserRole) {
    role.value = newRole
    saveUserToStorage()
  }
  
  // 设置当前房间
  function setCurrentRoom(roomId: string) {
    currentRoom.value = roomId
    saveUserToStorage()
  }
  
  // 退出登录
  function logout() {
    openid.value = ''
    nickname.value = ''
    avatar.value = ''
    isRegistered.value = false
    currentRoom.value = null
    currentGameId.value = null
    
    // 清除本地存储
    Taro.removeStorageSync('userInfo')
  }
  
  // 设置当前游戏
  function setCurrentGame(gameId: string | null) {
    currentGameId.value = gameId
  }
  
  // 重置用户角色到玩家
  function resetToPlayer() {
    role.value = 'player'
    saveUserToStorage()
  }
  
  // 清除房间和游戏信息
  function clearGameSession() {
    currentRoom.value = null
    currentGameId.value = null
    saveUserToStorage()
  }
  
  return {
    // 状态
    openid,
    nickname,
    avatar,
    role,
    isRegistered,
    currentRoom,
    currentGameId,
    loading,
    error,
    userInfo,
    
    // 方法
    initUser,
    registerUser,
    setRole,
    setCurrentRoom,
    logout,
    setCurrentGame,
    resetToPlayer,
    clearGameSession
  }
})